@@ -38,8 +38,8 @@ sub verif_Sheet1(;$){
   eq_deeply(title_rx(), 0) or die "${msg} title_rx is not 0";
   eq_deeply([@{ title_row() }],["First Name","Last Name","Email","Date"])
     or die "${msg} Sheet1 titles wrong";
-  #eq_deeply([@{ $rows[3] }],["Françoise-Athénaïs","de Rochechouart","","10/05/1640"])
-  eq_deeply([@{ $rows[3] }],["Françoise-Athénaïs","de Rochechouart","","Oct 5, 1640"])
+  eq_deeply([@{ $rows[3] }],["Françoise-Athénaïs","de Rochechouart","","10/05/1640"])
+  #eq_deeply([@{ $rows[3] }],["Françoise-Athénaïs","de Rochechouart","","Oct 5, 1640"])
     or confess "${msg} Sheet1 row 4 is wrong, got: ",avis(@{ $rows[3] });
 }
 sub verif_Another_Sheet(;$) {
@@ -88,8 +88,25 @@ my $got = [sort map{$_->basename} $dirpath->children];
 my $exp = [sort "Sheet1.csv", "Another Sheet.csv"];
 eq_deeply($got, $exp) or die dvis 'Missing or extra sheets: $got $exp';
 
-# "Read" a csv; should be a pass-thru without conversion
 my $testcsv_path = $dirpath->child("Sheet1.csv");
+my $exp_chars = $testcsv_path->slurp_utf8();
+
+# Convert csv to spreadsheet and back
+{
+  my $h1 = doconvert(inpath => $testcsv_path, cvt_to => "ods");
+  my $h2 = doconvert(inpath => $h1->{outpath}, cvt_to => "csv");
+  my $got_chars = path($h2->{outpath})->slurp_utf8;
+  if ($got_chars eq $exp_chars) {
+    say "Round-trip csv->ods->csv succeeded!\n" unless $silent;
+  } else {
+    $Data::Dumper::Interp::Foldwidth = 20;
+    die "Round-trip csv->ods->csv mismatch:\n",
+        ivis('Original: $exp_chars\n'),
+        ivis('Result:   $got_chars\n') ;
+  }
+}
+
+# "Read" a csv; should be a pass-thru without conversion
 {
   read_spreadsheet {verbose => $verbose}, $testcsv_path;
   verif_Sheet1 "(extracted csv)";
@@ -99,7 +116,6 @@ my $testcsv_path = $dirpath->child("Sheet1.csv");
 
 # csv-to-csv with transcoding
 {
-  my $exp_chars = $testcsv_path->slurp_utf8();
   my $tdir = Path::Tiny->tempdir();
   for my $enc (qw/UTF-8 UTF-16 UTF-32/) {
     say "------------- Transcode to/from $enc -------------" if $debug;
